@@ -1,5 +1,5 @@
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2007  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2007, 2008  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -62,7 +62,7 @@ f#,
 b,,
 >>> print n.get_user_octave_notename()
 b,,
->>> gettext.translation('solfege', './share/locale/', languages=['no_NO']).install()
+>>> gettext.translation('solfege', './share/locale/', languages=['nb_NO']).install()
 >>> print n.get_octave_notename()
 b,,
 >>> print n.get_user_octave_notename()
