@@ -1,5 +1,5 @@
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006, 2007  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -45,13 +45,14 @@ import mpdutils
 import _exceptions
 
 class ParseError(_exceptions.MpdException):
-    def __init__(self, toc):
-        self.m_toc = toc
-    def __str__(self):
-        return "Parse error: %s" % self.m_toc
+    def __init__(self, msg, lexer):
+        _exceptions.MpdException.__init__(self, msg)
+        self.m_lineno, self.m_linepos1, self.m_linepos2 = lexer.get_error_location()
 
 class LexerError(_exceptions.MpdException):
-    pass
+    def __init__(self, msg, lexer):
+        _exceptions.MpdException.__init__(self, msg)
+        self.m_lineno, self.m_linepos1, self.m_linepos2 = lexer.get_error_location()
 
 def musicalpitch_relative(first, second):
     """
@@ -438,11 +439,12 @@ class Score:
                     se.append(e)
             for voice in staff.m_voice_list:
                 for tie in voice.m_ties:
-                    sh1 = tie[3].m_shift
-                    sh2 = tie[4].m_shift
-                    se.append(TieEngraver(fontsize, tie[0], tie[1],
-                                          sh1, sh2,
-                                          clef.steps_to_ylinepos(tie[2][0])))
+                    if tie[0] in tv and tie[1] in tv:
+                        sh1 = tie[3].m_shift
+                        sh2 = tie[4].m_shift
+                        se.append(TieEngraver(fontsize, tie[0], tie[1],
+                                              sh1, sh2,
+                                              clef.steps_to_ylinepos(tie[2][0])))
         #########################################
         xv = {}
         p = 0
@@ -735,8 +737,13 @@ def parse_to_score_object(music):
     x = len(lexer.m_string)
     times = None
     cur_staff = None
-    while lexer.m_idx < x:
-        toc, toc_data = lexer.get()
+    while True:
+        try:
+            toc, toc_data = lexer.get()
+        except Exception, e:
+            if 'm_mpd_badcode' not in dir(e):
+                e.m_lineno, e.m_linepos1, e.m_linepos2 = lexer.get_error_location()
+            raise
         if toc == Lexer.STAFF:
             assert context == TOPLEVEL
             cur_staff = score.add_staff()
@@ -750,7 +757,7 @@ def parse_to_score_object(music):
             cur_staff.add_clef(timepos, "violin")
         elif toc == Lexer.VOICE:
             if not cur_staff:
-                raise ParseError("Don't use \\addvoice before \\staff")
+                raise ParseError("Don't use \\addvoice before \\staff", lexer)
             relative_mode = None
             timepos = Rat(0)
             cur_voice = cur_staff.add_voice()
@@ -776,10 +783,14 @@ def parse_to_score_object(music):
                 cur_voice.announce_timepos(timepos)
                 cur_voice.start_tuplet(timepos, times, tuplet_dir)
             else:
-                raise ParseError(r"\times nn/nn does not nest")
+                raise ParseError(r"\times nn/nn does not nest", lexer)
         elif toc == Lexer.CLEF:
             cur_staff.announce_timepos(timepos)
-            cur_staff.add_clef(timepos, toc_data)
+            try:
+                cur_staff.add_clef(timepos, toc_data)
+            except UnknownClefException, e:
+                e.m_lineno, e.m_linepos1, e.m_linepos2 = lexer.get_error_location()
+                raise
         elif toc == '|':
             cur_staff.announce_timepos(timepos)
             cur_staff.barline(timepos)
@@ -793,12 +804,12 @@ def parse_to_score_object(music):
                         k = ('c', 'major')
                     cur_staff.add_keysignature(Rat(0, 1), k)
             else:
-                raise ParseError("Token '{' not allowed here.")
+                raise ParseError("Token '{' not allowed here.", lexer)
         elif toc == '<':
             if context == NOTES:
                 context = START_OF_CHORD
             else:
-                raise ParseError("Token '<' not allowed here.")
+                raise ParseError("Token '<' not allowed here.", lexer)
         elif toc == '>':
             if context == CHORD:
                 if tie_is_in_the_air:
@@ -810,7 +821,7 @@ def parse_to_score_object(music):
                 relto = relto_backup; relto_backup = None
                 context = NOTES
             else:
-                raise ParseError("Token '>' not allowed here.")
+                raise ParseError("Token '>' not allowed here.", lexer)
         elif toc == '}':
             if context == NOTES:
                 if times:
@@ -819,7 +830,7 @@ def parse_to_score_object(music):
                 else:
                     context = TOPLEVEL
             else:
-                raise ParseError("Token '}' not allowed here.")
+                raise ParseError("Token '}' not allowed here.", lexer)
         elif toc == '[':
             cur_voice.announce_timepos(timepos)
             cur_voice.start_beam(timepos)
@@ -864,11 +875,10 @@ def parse_to_score_object(music):
             stem_dir = toc_data
         elif toc == Lexer.TUPLETDIR:
             tuplet_dir = toc_data
+        elif (toc, toc_data) == (None, None):
+            return score
         else:
-            print "*%s*" % toc, type(toc), lexer.m_idx
-            print "music: '%s'" % music[lexer.m_idx-4:]
-            raise ParseError(toc)
-    return score
+            raise ParseError(toc, lexer)
 
 
 class Lexer:
@@ -885,38 +895,47 @@ class Lexer:
     RELATIVE = 11
     TIMES = 12
     TUPLETDIR = 13
-    re_staff = re.compile(r"\s*\\staff", re.UNICODE)
-    re_voice = re.compile(r"\s*\\addvoice", re.UNICODE)
-    re_clef = re.compile(r"\s*\\clef\s+(\w*)", re.UNICODE)
-    re_clef_quoted = re.compile(r"\s*\\clef\s+\"([A-Za-z1-9]+[_^1-9]*)\"", re.UNICODE)
-    re_stem_updown = re.compile(r"\s*(\\stem)(Up|Down|Both)\s+", re.UNICODE)
-    re_tuplet_updown = re.compile(r"\s*(\\tuplet)(Up|Down|Both)\s+", re.UNICODE)
-    re_relative = re.compile(r"\s*\\relative\s+(([a-zA-Z]+)([',]*))", re.UNICODE)
-    re_transpose = re.compile(r"\s*\\transpose\s+(([a-zA-Z]+)([',]*))", re.UNICODE)
-    re_rest = re.compile(r"\s*(r)([\d]*)(\.*)", re.UNICODE)
+    re_staff = re.compile(r"\\staff", re.UNICODE)
+    re_voice = re.compile(r"\\addvoice", re.UNICODE)
+    re_clef = re.compile(r"\\clef\s+(\w*)", re.UNICODE)
+    re_clef_quoted = re.compile(r"\\clef\s+\"([A-Za-z1-9]+[_^1-9]*)\"", re.UNICODE)
+    re_stem_updown = re.compile(r"(\\stem)(Up|Down|Both)\s+", re.UNICODE)
+    re_tuplet_updown = re.compile(r"(\\tuplet)(Up|Down|Both)\s+", re.UNICODE)
+    re_relative = re.compile(r"\\relative\s+(([a-zA-Z]+)([',]*))", re.UNICODE)
+    re_transpose = re.compile(r"\\transpose\s+(([a-zA-Z]+)([',]*))", re.UNICODE)
+    re_rest = re.compile(r"(r)([\d]*)(\.*)", re.UNICODE)
         #FIXME we are a little more strict than Lilypond, since ~ has to
         # be before ]
         #FIXME don't use named regex if we don't need it.
     re_melodic = re.compile(r"""(?x)
-                            \s*
                              ((?P<notename>[a-zA-Z]+)
                              (?P<octave>[',]*))
                              (?P<len>[\d]*)
                              (?P<dots>\.*)""", re.UNICODE)
     re_skip = re.compile(r"""(?x)
-                            \s*
                              (s)
                              (?P<len>[\d]*)
                              (?P<dots>\.*)""", re.UNICODE)
-    re_tempo = re.compile(r"\s*\\tempo\s+(\d+)\s*=+s*(\d+);", re.UNICODE)
-    re_time = re.compile(r"\s*\\time\s+(\d+)\s*/\s*(\d+)", re.UNICODE)
-    re_key = re.compile(r"\s*\\key\s+([a-z]+)\s*\\(major|minor)", re.UNICODE)
-    re_times = re.compile(r"\s*\\times\s+(\d+)\s*/\s*(\d+)\s*{", re.UNICODE)
+    re_time = re.compile(r"\\time\s+(\d+)\s*/\s*(\d+)", re.UNICODE)
+    re_key = re.compile(r"\\key\s+([a-z]+)\s*\\(major|minor)", re.UNICODE)
+    re_times = re.compile(r"\\times\s+(\d+)\s*/\s*(\d+)\s*{", re.UNICODE)
     def __init__(self, s):
-        self.m_string = s.strip()
+        self.m_string = s
         self.m_notelen = Duration(4, 0)
         self.m_idx = 0
+        self.m_last_idx = None
     def get(self):
+        """
+        return None, None when there are no more tokens.
+        """
+        # Doing this while loop inside the exception clause is a little
+        # faster than using a regular expression.
+        try:
+            while self.m_string[self.m_idx] in (' ', '\n', '\t'):
+                self.m_idx += 1
+        except IndexError:
+            pass
+        self.m_last_idx = self.m_idx
         m = self.re_rest.match(self.m_string, self.m_idx)
         if m:
             self.m_idx += len(m.group())
@@ -930,11 +949,11 @@ class Lexer:
                     raise LexerError('Need a digit before dots. Write "%(goodcode)s", not "%(badcode)s".' % {
                         'badcode': m.group().strip(),
                         'goodcode':'%s%i%s' % (resttype, self.m_notelen.m_nh, dots)
-                        })
+                        },
+                        self)
             r = RestRequest(notelen, numdots)
             if not r.m_duration:
                 r.m_duration = self.m_notelen.clone()
-             #   if numdots:
                 self.m_notelen.m_dots = numdots
             else:
                 self.m_notelen = r.m_duration.clone()
@@ -944,14 +963,16 @@ class Lexer:
         if m:
             self.m_idx += len(m.group())
             IGN1, skiplen, dots = m.groups()
+            numdots = len(dots)
             if skiplen:
                 skiplen = int(skiplen)
             else:
                 skiplen = 0
-                if dots:
-                    #FIXME
-                    raise ParseError("Error")
-            numdots = len(dots)
+                if numdots:
+                    raise LexerError('Need a digit before dots. Write "%(goodcode)s", not "%(badcode)s".' % {
+                        'badcode': m.group().strip(),
+                        'goodcode':'s%i%s' % (self.m_notelen.m_nh, dots)
+                        }, lexer)
             if skiplen is 0:
                 retval = self.m_notelen
             else:
@@ -970,7 +991,7 @@ class Lexer:
                     raise LexerError('Need a digit before dots. Write "%(goodcode)s", not "%(badcode)s".' % {
                         'badcode': m.group().strip(),
                         'goodcode':'%s%i%s' % (notename, self.m_notelen.m_nh, dots)
-                        })
+                        }, self)
             n = MusicRequest(notename, notelen, numdots)
             if not n.m_duration:
                 n.m_duration = self.m_notelen.clone()
@@ -1023,7 +1044,26 @@ class Lexer:
         if m:
             self.m_idx += len(m.group())
             return self.TIMES, Rat(int(m.groups()[0]), int(m.groups()[1]))
-        while self.m_string[self.m_idx] in [' ', '\n', '\t']:
-            self.m_idx += 1
+        if self.m_idx == len(self.m_string):
+            return None, None
         self.m_idx += 1
         return self.m_string[self.m_idx-1], None
+    def get_error_location(self):
+        """
+        Return a tuple
+        (lineno, pos1, pos2)
+        lineno is the 0-index line where the error occoured.
+        string[pos1:pos2] will return the exact text that caused the error.
+        """
+        # Let us first count lines to find which line we are on.
+        # line numbers are 0-indexed
+        lineno = self.m_string[:self.m_last_idx].count("\n")
+        line_start = self.m_last_idx
+        while line_start > 0 and self.m_string[line_start] != "\n":
+            line_start -= 1
+        if self.m_string[line_start] == "\n":
+            line_start += 1
+        line_end = line_start
+        while line_end < len(self.m_string) and self.m_string[line_end] != '\n':
+            line_end += 1
+        return (lineno, self.m_last_idx - line_start, self.m_idx - line_start)
