@@ -1,5 +1,5 @@
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006, 2007  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -43,10 +43,6 @@ _saved_synth = None
 if sys.platform == 'win32':
     import winsound
 
-class SoundSetupException(Exception):
-    def __init__(self, msg):
-        Exception.__init__(self, _("Sound setup error"), msg)
-
 def play_mediafile(typeid, filename):
     if sys.platform == 'win32' and typeid == 'wav':
         winsound.PlaySound(filename, winsound.SND_FILENAME | winsound.SND_ASYNC)
@@ -56,7 +52,8 @@ def play_mediafile(typeid, filename):
             run_external_program(
                 src.cfg.get_string("sound/%s_player" % typeid), "", filename)
         except OsUtilsException, e:
-            raise SoundSetupException(_("Check the config for %(typeid)s files in the preferences window (Ctrl-F12). %(msg)s") % {'typeid': typeid, 'msg': str(e)})
+            e.args = e.args + (_("Check the config for %(typeid)s files in the preferences window (Ctrl-F12).") % locals(),)
+            raise
 
 
 def initialise_winsynth(synthnum, verbose_init=0):
