@@ -1,5 +1,5 @@
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -482,22 +482,31 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         spin.set_sensitive(b.get_active())
         hbox.pack_start(spin, False)
     def _lessonfile_exception(self, exception, sourcefile, lineno):
-        if self.m_t.m_P._idx is None:
-            msg = ""
-        else:
-            msg = _('Please check question number %(idx)i in the lesson file "%(lf)s".\n\n') % {'idx': self.m_t.m_P._idx+1, 'lf': self.m_t.m_P.m_filename}
-        msg += _('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': sourcefile, 'lineno': lineno}
         m = ExceptionDialog(exception)
-        m.add_text(msg)
+        try:
+            idx = exception.m_question_idx
+        except AttributeError:
+            idx = self.m_t.m_P._idx
+        if idx is not None:
+            m.add_text(_('Please check question number %(idx)i in the lesson file "%(lf)s".') % {'idx': self.m_t.m_P._idx+1, 'lf': self.m_t.m_P.m_filename})
         if 'm_nonwrapped_text' in dir(exception):
             m.add_nonwrapped_text(exception.m_nonwrapped_text)
+        if 'm_mpd_badcode' in dir(exception) and exception.m_mpd_badcode:
+            # some music classes may return an empty string if they have
+            # nothing useful to display.
+            m.add_nonwrapped_text(exception.m_mpd_badcode)
+        m.add_text(_('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': sourcefile, 'lineno': lineno})
         m.run()
         m.destroy()
     def _mpd_exception(self, exception, sourcefile, lineno):
+        try:
+            idx = exception.m_question_idx + 1
+        except AttributeError:
+            idx = self.m_t.m_P._idx + 1
         m = ExceptionDialog(exception)
         if 'm_mpd_varname' in dir(exception):
             m.add_text(_('Failed to parse the music in the variable "%(varname)s" in question number %(idx)i in the lesson file "%(lf)s".') % {
-                'idx': self.m_t.m_P._idx+1, 
+                'idx': idx,
                 'lf': self.m_t.m_P.m_filename, 
                 'varname': exception.m_mpd_varname})
         else:
@@ -513,14 +522,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         """
         try:
             return method(*args, **kwargs)
-        except lessonfile.LessonfileException, e:
-            t = traceback.extract_stack()
-            self._lessonfile_exception(e, t[-2][0], t[-2][1])
-        except mpd.MpdException, e:
-            t = traceback.extract_stack()
-            self._mpd_exception(e, t[-2][0], t[-2][1])
         except Exception, e:
-            t = traceback.extract_stack()
             if not self.standard_exception_handler(e):
                 raise
     def standard_exception_handler(self, e, cleanup_function=lambda: False):
@@ -544,7 +546,11 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         sourcefile = sourcefile.decode(sys.getfilesystemencoding(), 'replace')
         if self.m_t.m_app.m_options.disable_exception_handler:
             return False
-        if isinstance(e, lessonfile.LessonfileException):
+        elif isinstance(e, lessonfile.NoQuestionsConfiguredException):
+            cleanup_function()
+            self.g_win.display_error_message2(e.args[0], e.args[1])
+            return True
+        elif isinstance(e, lessonfile.LessonfileException):
             cleanup_function()
             self._lessonfile_exception(e, sourcefile, lineno)
             return True
@@ -552,7 +558,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             cleanup_function()
             self._mpd_exception(e, sourcefile, lineno)
             return True
-        elif isinstance(e, (osutils.OsUtilsException, soundcard.SoundSetupException)):
+        elif isinstance(e, osutils.OsUtilsException):
             cleanup_function()
             self.g_win.display_exception_message(e)
             return True
@@ -757,7 +763,7 @@ class LessonbasedGui(Gui):
             varname = 'vmusic'
         else:
             varname = 'music'
-        if not self.m_t.m_P.get_question()[varname].is_displayable():
+        if not isinstance(self.m_t.m_P.get_question()[varname], lessonfile.MpdDisplayable):
             return
         self.display_music(varname)
     def display_music(self, varname):
@@ -774,16 +780,16 @@ class LessonbasedGui(Gui):
                 self.g_win.display_in_musicviewer(self.m_t.m_P.get_music(varname))
         except mpd.MpdException, e:
             e.m_mpd_varname = varname
-            e.m_mpd_badcode = self.m_t.m_P.get_question()[varname].m_musicdata
+            e.m_mpd_badcode = self.m_t.m_P.get_question()[varname].get_err_context(e, self.m_t.m_P)
             if not self.standard_exception_handler(e):
-                raise e
+                raise
     def do_at_question_start_show_play(self):
         """
         This method is shared by idbyname and elembuilder, and possibly
         other exercises later. It will show and/or play music based on
         the header.at_question_start  variable.
 
-        It might raise mpd.MpdExceptions.
+        It might raise mpd.MpdException.
         """
         if self.m_t.m_P.header.at_question_start == 'show':
             self.show_answer()
