@@ -1,5 +1,5 @@
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -207,10 +207,17 @@ _("""You have to select some tones practise. Do this on the config page by setti
             self.g_new_tone.set_sensitive(
                   not self.get_bool('config/picky_on_new_question'))
             self.g_repeat.set_sensitive(True)
-            if self.m_t.m_is_first_question:
-                self.flash_and_play_first_tone()
-            self.g_flashbar.clear()
-            self.m_t.play_question()
+            try:
+                if self.m_t.m_is_first_question:
+                    self.flash_and_play_first_tone()
+                else:
+                    self.g_flashbar.clear()
+                    self.m_t.play_question()
+            except Exception,e:
+                def cleanup():
+                    self.g_repeat.set_sensitive(False)
+                if not self.standard_exception_handler(e, cleanup):
+                    raise
         self.set_percentage_label()
     def flash_and_play_first_tone(self):
         self.g_flashbar.flash(_("First tone is %s") % self.m_t.m_question)
@@ -230,19 +237,23 @@ _("""You have to select some tones practise. Do this on the config page by setti
                 self.g_new_tone.set_sensitive(True)
                 self.g_give_up.set_sensitive(False)
             else:
-                if self.m_t.m_is_first_question:
-                    self.flash_and_play_first_tone()
-                    return
-                self.g_flashbar.flash(_("Wrong"))
-                self.g_give_up.set_sensitive(True)
-                if self.get_bool("warning_sound"):
-                    if self.get_bool("config/auto_repeat_question_if_wrong_answer"):
-                        self.m_t.spank_me_play_question()
+                try:
+                    if self.m_t.m_is_first_question:
+                        self.flash_and_play_first_tone()
+                        return
+                    self.g_flashbar.flash(_("Wrong"))
+                    self.g_give_up.set_sensitive(True)
+                    if self.get_bool("warning_sound"):
+                        if self.get_bool("config/auto_repeat_question_if_wrong_answer"):
+                            self.m_t.spank_me_play_question()
+                        else:
+                            self.m_t.spank_me()
                     else:
-                        self.m_t.spank_me()
-                else:
-                    if self.get_bool("config/auto_repeat_question_if_wrong_answer"):
-                        self.m_t.play_question()
+                        if self.get_bool("config/auto_repeat_question_if_wrong_answer"):
+                            self.m_t.play_question()
+                except Exception, e:
+                    if not self.standard_exception_handler(e):
+                        raise
         self.set_percentage_label()
     def give_up(self, _o=None):
         if self.m_t.q_status == self.QSTATUS_WRONG:
