@@ -1,5 +1,5 @@
 # GNU Solfege - free ear training software
-# Copyright (C) 2005, 2006, 2007  Tom Cato Amundsen
+# Copyright (C) 2005, 2006, 2007, 2008  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -700,7 +700,7 @@ class Window(gtk.Window):
         self.m_modified = False
         self.show_all()
         self.treeview_cursor_changed()
-        self.__load_file2_lock = 0
+        self.__load_file_lock = 0
     def fill_trees_combo(self):
         self.m_trees = [('solfege', 'learningtree.txt', os.path.join(os.getcwdu(), 'learningtree.txt'))]
         if self.m_app.m_options.debug:
@@ -716,28 +716,28 @@ class Window(gtk.Window):
             self.g_trees_liststore.append((txt,))
     def set_trees_combo(self, place, filename):
         # Set the combo to the correct learning tree.
-        self.__load_file2_lock += 1
+        self.__load_file_lock += 1
         idx = 0
         while idx < len(self.m_trees):
             if self.m_trees[idx][0] == place \
                     and self.m_trees[idx][1] == filename:
                 self.g_trees.set_active(idx)
             idx += 1
-        self.__load_file2_lock -= 1
+        self.__load_file_lock -= 1
     def on_learning_tree_combo_changed(self, combobox):
         t, fn, txt = self.m_trees[combobox.get_active()]
         if self.m_tree.m_modified:
             if gu.dialog_yesno(_("The file is not saved. Save before changing?"), self):
                 self._do_save(self.m_filename)
-        if not self.__load_file2_lock:
-            self.load_file2(t, fn)
+        if not self.__load_file_lock:
+            self.load_file(t, fn)
         self.treeview_cursor_changed()
         cfg.set_list('app/learningtree', [t, fn])
         self.m_app.m_ui.on_learning_tree_changed()
     def treeview_cursor_changed(self, w=None):
         path = self.g_treeview.get_cursor()[0]
         self.g_new_submenu.set_sensitive(path is not None and len(path) < 2)
-        self.g_add_lesson.set_sensitive(path is not None and len(path) == 2)
+        self.g_add_lesson.set_sensitive(path is not None and len(path) >= 2)
         self.g_delete_lesson.set_sensitive(path is not None)
         self.g_move_lesson_up.set_sensitive(path is not None and len(path) == 3)
         self.g_move_lesson_down.set_sensitive(path is not None and len(path) == 3)
@@ -836,7 +836,7 @@ class Window(gtk.Window):
 """)
                         f.close()
                         self.fill_trees_combo()
-                        self.load_file2('user', filename)
+                        self.load_file('user', filename)
                         cfg.set_list('app/learningtree', ['user', filename])
                         self.m_app.m_ui.on_learning_tree_changed()
                     except IOError, e:
@@ -1109,6 +1109,10 @@ class Window(gtk.Window):
             dlg.destroy()
     def on_new_lesson(self, *v):
         path = self.g_treeview.get_cursor()[0]
+        # By trunkating path, we can add lessons to the submenu we are on
+        # when a lesson is selected.
+        if len(path) == 3:
+            path = path[:2]
         dlg = SelectLessonFileDialog(self.m_app, self.m_tree, path)
         dlg.run()
         lesson_id = dlg.get_selected_lesson_id()
@@ -1178,7 +1182,7 @@ class Window(gtk.Window):
             iter = self.m_model.get_iter(path)
             self.m_model.swap(iter, self.m_model.get_iter(next_path))
             self.g_treeview.scroll_to_cell(next_path)
-    def load_file2(self, place, filename):
+    def load_file(self, place, filename):
         """
         place is 'solfege' or 'user'
         filename is a filename, with no directory part.
@@ -1197,6 +1201,7 @@ class Window(gtk.Window):
                     gtk.BUTTONS_CLOSE, _("The default learning tree is write protected in your install. This is normal. If you want to edit a learning tree, you have to select one of the trees stored in .solfege/learningtrees in your home directory."))
                 m.run()
                 m.destroy()
+        self.g_treeview.grab_focus()
     def _load_file(self, filename):
         self.m_tree = LearningTree(self.m_app.lessonfile_manager)
         # We use _m_orig_tree just to see if things has changed.
