@@ -1,5 +1,5 @@
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007 Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,12 +25,27 @@ import stat
 import time
 import locale
 import filesystem
+import utils
+
+import re, os, sys
+import random
+import cfg
+import mpd
+from mpd import mpdutils
+from mpd.rat import Rat
+import soundcard
+import dataparser
+from dataparser import istr
+import osutils
 
 _test_mode = False
 
 class LessonfileException(Exception):
     pass
 
+class MusicObjectException(mpd.MpdException):
+    pass
+
 class LessonfileParseException(Exception):
     pass
 
@@ -40,33 +55,22 @@ class NoQuestionsInFileException(LessonfileParseException):
     no questions at all.
     """
     def __init__(self, lessonfilename):
-        self.m_filename = lessonfilename
-    def __str__(self):
-        return "The lesson file '%s' contains no questions." % self.m_filename
+        LessonfileParseException.__init__(self, _("The lesson file contains no questions"))
 
 class FileNotFound(LessonfileException):
     def __init__(self, filename):
-        self.m_filename = filename
-    def __str__(self):
-        return "The external file '%s' was not found." % self.m_filename
+        LessonfileException.__init__(self, _("The external file '%s' was not found") % filename)
 
-class NoQuestionConfiguredException(LessonfileException):
+class NoQuestionsConfiguredException(LessonfileException):
     """
     This exception is raised by select_random_question if the user has
     unselected all the questions available in the lesson file.
     """
-    def __str__(self):
-        return "All questions are unselected. Select some questions from the config page."
+    def __init__(self):
+        LessonfileException.__init__(self,
+            _("No questions selected"),
+            _("You can select questions on the config page of the exercise."))
 
-import re, os, sys
-import random
-import mpd
-import cfg
-from mpd.rat import Rat
-import soundcard
-import dataparser
-from dataparser import istr
-import osutils
 
 class plstring(unicode):
     pass
@@ -119,6 +123,7 @@ keywords = (
     'elembuilder',
     'rhythmtapping',
     'rhythmtapping2',
+    'idproperty',
     #
     'harmonic',
     'melodic',
@@ -199,11 +204,6 @@ class _Header(dict):
                            ('have_music_displayer', False),
                            ('enable_right_click', True),
                            ('disable_unused_intervals', True),
-                           ('qprops', ['name', 'inversion', 'toptone']),
-                           ('qprop_labels', [
-                             istr.new_translated("Chord type", _("Chord type")),
-                             istr.new_translated("Inversion", _("Inversion")),
-                             istr.new_translated("Toptone", _("Toptone"))]),
                            ):
             if key not in self:
                 self[key] = value
@@ -225,37 +225,60 @@ class _Header(dict):
 class MusicBaseClass(object):
     def __init__(self, musicdata):
         self.m_musicdata = musicdata
-    def is_mpd_parsable(self):
-        return False
-    def is_displayable(self):
-        return False
-    def is_transposable(self):
-        return False
-    def play(self, lessonfile_ref, question):
-        assert False, "%s does not implement play(...)" % self.__class__.__name__
-    def play_arpeggio(self, lessonfile_ref, question):
-        assert False, "%s does not implement play_arpeggio(...)" % self.__class__.__name__
     def get_mpd_music_string(self, lessonfile_ref):
         return "%s:%s" % (self.__class__.__name__, self.m_musicdata)
-    def get_music_as_notename_list(self, lessonfile_ref):
-        assert False, "%s does not implement get_music_as_notename_list(...)" % self.__class__.__name__
 
 
 class MpdParsable(MusicBaseClass):
-    def is_mpd_parsable(self):
-        return True
+    """
+    MpdParsable implements two generic play and play_slowly methods
+    that can play the music if the subclass implements get_mpd_music_string.
+    Music classes with more special needs will overwrite these play methods.
+    """
+    def play(self, lessonfile_ref, question):
+        assert isinstance(lessonfile_ref, LessonfileCommon)
+        instrument = lessonfile_ref.prepare_instrument_list(question)
+        mpd.play_music(self.get_mpd_music_string(lessonfile_ref),
+            lessonfile_ref.get_tempo(), instrument[0], instrument[1])
+        if _test_mode:
+            return self.get_mpd_music_string(lessonfile_ref)
+    def play_slowly(self, lessonfile_ref, question):
+        assert isinstance(lessonfile_ref, LessonfileCommon)
+        instrument = lessonfile_ref.prepare_instrument_list(question)
+        tempo = lessonfile_ref.get_tempo()
+        tempo = (tempo[0]/2, tempo[1])
+        mpd.play_music(self.get_mpd_music_string(lessonfile_ref),
+            tempo, instrument[0], instrument[1])
+        if _test_mode:
+            return self.get_mpd_music_string(lessonfile_ref)
+    def get_err_context(self, exception, lessonfile_ref):
+        """
+        Return a twoline string showing what caused the exception.
+        """
+        # I had to comment out the assert statement below, becuase the rhythmtapping
+        # exercise will analyze the m_musicdata string directly and not the data from
+        # get_mpd_music_string. And then the error can be on line 0.
+        #assert exception.m_lineno != 0, "This nevers happend"
+        first = exception.m_linepos1
+        last = exception.m_linepos2
+        if self.m_musicdata.count("\n") + 1 < exception.m_lineno:
+            return "(FIXME: we need better error reporting)\nBad input to the music object of type %s causes the\nfollowing generated music code:" % self.__class__.__name__.lower() + "\n" +  self.get_mpd_music_string(lessonfile_ref)
+        return "\n".join((self.m_musicdata.split("\n")[exception.m_lineno-1],
+            " " * first + "^" * (last - first)))
 
 
-class PitchBaseClass(MpdParsable):
-    def is_displayable(self):
-        return True
+class MpdDisplayable(MpdParsable):
+    pass
 
+class MpdTransposable(MpdDisplayable):
+    pass
 
-class Chord(PitchBaseClass):
+class ChordCommon(MpdTransposable):
+    pass
+
+class Chord(ChordCommon):
     def __init__(self, musicdata):
-        PitchBaseClass.__init__(self, musicdata)
-    def is_transposable(self):
-        return True
+        ChordCommon.__init__(self, musicdata)
     def get_lilypond_code(self, lessonfile_ref):
         return r"\score{ "\
                r" \transpose c' %s{ <%s> }"\
@@ -273,18 +296,25 @@ class Chord(PitchBaseClass):
     def get_mpd_music_string(self, lessonfile_ref):
         assert isinstance(lessonfile_ref, LessonfileCommon)
         if lessonfile_ref.header.random_transpose:
-            return r"\staff\transpose %s{ <%s> }" \
+            return "\\staff\\transpose %s{<\n%s\n>}" \
                 % (lessonfile_ref.m_transpose.get_octave_notename(),
                    self.m_musicdata)
-        return r"\staff{ <%s> }" % self.m_musicdata
+        return "\\staff{<\n%s\n>}" % self.m_musicdata
     def get_music_as_notename_list(self, lessonfile_ref):
+        """
+        This method will validate the notenames, and raise a
+        mpd.musicalpitch.InvalidNotenameException with the
+        m_linepos1, m_linepos2 og m_lineno.
+        """
         assert isinstance(lessonfile_ref, LessonfileCommon)
-        if not lessonfile_ref.header.random_transpose:
-            return self.m_musicdata.split()
-        v = []
-        for n in self.m_musicdata.split():
-            v.append(mpd.MusicalPitch.new_from_notename(n).transpose_by_musicalpitch(lessonfile_ref.m_transpose).get_octave_notename())
-        return v
+        try:
+            if not lessonfile_ref.header.random_transpose:
+                return [mpd.MusicalPitch.new_from_notename(n).get_octave_notename() for n in self.m_musicdata.split()]
+            else:
+                return [mpd.MusicalPitch.new_from_notename(n).transpose_by_musicalpitch(lessonfile_ref.m_transpose).get_octave_notename() for n in self.m_musicdata.split()]
+        except mpd.InvalidNotenameException, e:
+            e.m_lineno, e.m_linepos1, e.m_linepos2 = mpdutils.validate_only_notenames(self.m_musicdata)
+            raise
     def get_music_as_notename_string(self, lessonfile_ref):
         return " ".join(self.get_music_as_notename_list(lessonfile_ref))
     def play(self, lessonfile_ref, question):
@@ -350,52 +380,14 @@ class Chord(PitchBaseClass):
             soundcard.synth.play_track(t1, t2, t3)
 
 
-class _VoiceCommon(PitchBaseClass):
-    def is_transposable(self):
-        return True
-    def play(self, lessonfile_ref, question):
-        assert isinstance(lessonfile_ref, LessonfileCommon)
-        instrument = lessonfile_ref.prepare_instrument_list(question)
-        mpd.play_music(self.get_mpd_music_string(lessonfile_ref),
-            lessonfile_ref.get_tempo(), instrument[0], instrument[1])
-        if _test_mode:
-            return self.get_mpd_music_string(lessonfile_ref)
-    def play_slowly(self, lessonfile_ref, question):
-        assert isinstance(lessonfile_ref, LessonfileCommon)
-        instrument = lessonfile_ref.prepare_instrument_list(question)
-        tempo = lessonfile_ref.get_tempo()
-        tempo = (tempo[0]/2, tempo[1])
-        mpd.play_music(self.get_mpd_music_string(lessonfile_ref),
-            tempo, instrument[0], instrument[1])
-        if _test_mode:
-            return self.get_mpd_music_string(lessonfile_ref)
-    def find_first_note(self):
-        r"""
-        Return the index of of the first note from self.m_musicdata.split()
-        We will use this note to do \relative mode.
-        Return -1 if no notes are found.
-        """
-        i = 0
-        v = self.m_musicdata.split()
-        while 1:
-            n = re.match("(\<?)[a-zA-Z]+[',]*", v[i])
-            if n:
-                break
-            if v[i] == r"\clef" or v[i] == r"\time":
-                i += 2
-            elif v[i] == r"\key":
-                i += 3
-            else:
-                i += 1
-            if i >= len(v):
-                return -1
-        return i
+class VoiceCommon(MpdTransposable):
+    def get_first_pitch(self):
+        a, b = mpdutils.find_possible_first_note(self.m_musicdata)
+        return self.m_musicdata[a:b]
 
-
-
-class Voice(_VoiceCommon):
+class Voice(VoiceCommon):
     def __init__(self, musicdata):
-        _VoiceCommon.__init__(self, musicdata)
+        VoiceCommon.__init__(self, musicdata)
     def get_lilypond_code(self, lessonfile_ref):
         return r"\transpose c' %s{ %s }" % (
             lessonfile_ref.m_transpose.get_octave_notename(),
@@ -409,42 +401,54 @@ class Voice(_VoiceCommon):
                r" \layout { ragged-last = ##t } " \
                r"}" % (
             lessonfile_ref.m_transpose.get_octave_notename(),
-            self.m_musicdata.split()[self.find_first_note()],
+            self.get_first_pitch(),
             self.m_musicdata,
             )
     def get_mpd_music_string(self, lessonfile_ref):
         assert isinstance(lessonfile_ref, LessonfileCommon)
         if lessonfile_ref.header.random_transpose:
-            return r"\staff\transpose %s{ %s }" \
+            return "\\staff\\transpose %s{\n%s\n}" \
                 % (lessonfile_ref.m_transpose.get_octave_notename(),
                    self.m_musicdata)
-        return r"\staff{ %s }" % self.m_musicdata
+        return "\\staff{\n%s\n}" % self.m_musicdata
 
-
-class Rvoice(_VoiceCommon):
+class Rvoice(VoiceCommon):
     def __init__(self, musicdata):
-        _VoiceCommon.__init__(self, musicdata)
+        VoiceCommon.__init__(self, musicdata)
+    def get_err_context(self, exception, lessonfile_ref):
+        """
+        Return a twoline string showing what caused the exception.
+        """
+        if exception.m_lineno == 0:
+            # If there is an error in the first pitch, this will cause the
+            # mpd parser to raise and exception on the \relative XXX line,
+            # and this is on line 0.
+            a, b = mpdutils.find_possible_first_note(self.m_musicdata)
+            line = utils.string_get_line_at(self.m_musicdata, a)
+            a, b = mpdutils.find_possible_first_note(line)
+            return "\n".join((line, " " * (a) + "^" * (b - a)))
+        else:
+            return VoiceCommon.get_err_context(self, exception, lessonfile_ref)
     def get_mpd_music_string(self, lessonfile_ref):
         assert isinstance(lessonfile_ref, LessonfileCommon)
-        i = self.find_first_note()
+        a, b = mpdutils.find_possible_first_note(self.m_musicdata)
+        if (a, b) != (None, None):
+            first_pitch = self.m_musicdata[a:b]
+        else:
+            first_pitch = None
         if lessonfile_ref.header.random_transpose:
             transpose_str = r"\transpose %s" % lessonfile_ref.m_transpose.get_octave_notename()
         else:
             transpose_str = ""
-        if i == -1:
+        if not first_pitch:
             # Returning there is mostly theoretical. Why would anyone
             # have a music object with no notes, only commands like
             # \clef, \staff etc?
-            return r"\staff%s{ %s }"\
+            return "\\staff%s{\n%s\n}"\
                % (transpose_str, self.m_musicdata)
-        v = self.m_musicdata.split()
-        m = re.match("(?P<pre>\<?)(?P<note>[a-zA-Z]+[',]*)", v[i])
-        rel_note = m.group('note')
-        nn = mpd.MusicalPitch.new_from_notename(rel_note)
-        aa = m.group('pre') + nn.get_octave_notename()
-        v[i] = m.group('pre') + nn.get_notename() + v[i][len(aa):]
-        return r"\staff%s\relative %s{ %s }" \
-          % (transpose_str, rel_note, " ".join(v))
+        new_musicdata = self.m_musicdata[:a] + first_pitch.rstrip(",'") + self.m_musicdata[b:]
+        return "\\staff%s\\relative %s{\n%s\n}" \
+          % (transpose_str, first_pitch, new_musicdata)
     def get_lilypond_code(self, lessonfile_ref):
         return r"\transpose c' %s\relative c{ %s }" % (
             lessonfile_ref.m_transpose.get_octave_notename(),
@@ -458,16 +462,23 @@ class Rvoice(_VoiceCommon):
                r" \layout { ragged-last = ##t } " \
                r"}" % (
             lessonfile_ref.m_transpose.get_octave_notename(),
-            self.m_musicdata.split()[self.find_first_note()],
+            self.get_first_pitch(),
             self.m_musicdata,
             )
 
-class Satb(_VoiceCommon):
+class Satb(ChordCommon):
     def __init__(self, musicdata):
-        _VoiceCommon.__init__(self, musicdata)
+        ChordCommon.__init__(self, musicdata)
+        if "\n" in self.m_musicdata:
+            self._m_orig_musicdata = self.m_musicdata
+            self.m_musicdata = self.m_musicdata.replace("\n", "")
     def get_mpd_music_string(self, lessonfile_ref):
         assert isinstance(lessonfile_ref, LessonfileCommon)
         v = [n.strip() for n in self.m_musicdata.split('|')]
+        if len(v) != 4:
+            raise MusicObjectException("Satb music should be divided into 4 parts by the '|' character")
+        if [x for x in self.m_musicdata.split("|") if not x.strip()]:
+            raise MusicObjectException("Satb music does not allow an empty voice")
         #FIXME BUG BUG BUG this only works for the currently active question
         if 'key' in lessonfile_ref.get_question():
             k = lessonfile_ref.get_question()['key']
@@ -483,13 +494,6 @@ class Satb(_VoiceCommon):
             music = music.replace(r"\addvoice",
                       r"\addvoice\transpose %s" % lessonfile_ref.m_transpose.get_octave_notename())
         return music
-    def play(self, lessonfile_ref, question):
-        # We don't want this method to work, since only play_arpeggio
-        # does octave transposition based on the users gender, and that
-        # only play_arpeggio is used by singinterval
-        assert False, "Satb does by design not implement play(...)"
-    def play_slowly(self, lessonfile_ref, question):
-        assert False, "Satb does by design not implement play_slowly(...)"
     def play_arpeggio(self, lessonfile_ref, question):
         assert isinstance(lessonfile_ref, LessonfileCommon)
         instrument = lessonfile_ref.prepare_instrument_list(question)
@@ -516,11 +520,35 @@ class Satb(_VoiceCommon):
                 else:
                     track.note(4, mpd.notename_to_int(n)+12, instrument[1])
         soundcard.synth.play_track(track)
+    def get_err_context(self, exception, lessonfile_ref):
+        """
+        Return a twoline string showing what caused the exception.
+        """
+        if len(self.m_musicdata.split("|")) != 4:
+            return self.m_musicdata
+        if [x for x in self.m_musicdata.split("|") if not x.strip()]:
+            return self.m_musicdata
+        line1 = []
+        line2 = []
+        err_found = False
+        try:
+            if self._m_orig_musicdata:
+                bad_err_msg = "\n".join(textwrap.wrap("The music code from the lesson file has been modified by removing the new-line characters. This to more easily show where the error occured. Satb music should not contain music characters.", 60)) + "\n"
+        except AttributeError:
+            bad_err_msg = ""
+        for i, s in enumerate(self.m_musicdata.split("|")):
+            line1.append(s)
+            if i == exception.m_lineno:
+                line2.append("^" * len(s))
+                err_found = True
+            elif not err_found:
+                line2.append(" " * len(s))
+        return bad_err_msg + ("\n".join(("|".join(line1), " ".join(line2))))
 
 
-class _PercBaseClass(MpdParsable):
+class PercBaseClass(MpdParsable):
     def get_mpd_music_string(self, lessonfile_ref):
-        return r"\staff{ %s }" % self.m_musicdata
+        return "\\staff{\n%s\n}" % self.m_musicdata
     def _gen_track(self, lessonfile_ref, question):
         score = mpd.parser.parse_to_score_object(self.get_mpd_music_string(lessonfile_ref))
         track = score.get_midi_events_as_percussion(cfg.get_int('config/preferred_instrument_velocity'))[0]
@@ -529,9 +557,9 @@ class _PercBaseClass(MpdParsable):
         return track
 
 
-class Rhythm(_PercBaseClass):
+class Rhythm(PercBaseClass):
     def __init__(self, musicdata):
-        _PercBaseClass.__init__(self, musicdata)
+        PercBaseClass.__init__(self, musicdata)
     def play(self, lessonfile_ref, question):
         track = self._gen_track(lessonfile_ref, question)
         track.replace_note(mpd.notename_to_int("c"), 37)
@@ -539,9 +567,9 @@ class Rhythm(_PercBaseClass):
         soundcard.synth.play_track(track)
 
 
-class Percussion(_PercBaseClass):
+class Percussion(PercBaseClass):
     def __init__(self, musicdata):
-        _PercBaseClass.__init__(self, musicdata)
+        PercBaseClass.__init__(self, musicdata)
     def play(self, lessonfile_ref, question):
         soundcard.synth.play_track(self._gen_track(lessonfile_ref, question))
 
@@ -556,6 +584,8 @@ class _MusicExternalPlayer(MusicBaseClass):
             soundcard.play_mediafile(self.m_typeid, musicfile)
         else:
             raise FileNotFound(musicfile)
+    def get_err_context(self, exception, lessonfile_ref):
+        return ""
 
 class Midifile(_MusicExternalPlayer):
     def __init__(self, musicdata):
@@ -583,9 +613,9 @@ class Cmdline(MusicBaseClass):
                                      lessonfile_ref.m_location, "")
 
 
-class Music(_VoiceCommon):
+class Music(MpdDisplayable):
     def __init__(self, musicdata):
-        _VoiceCommon.__init__(self, musicdata)
+        MpdDisplayable.__init__(self, musicdata)
     def get_mpd_music_string(self, lessonfile_ref):
         if lessonfile_ref.header.random_transpose:
             s = self.m_musicdata.replace(r'\staff',
@@ -597,14 +627,14 @@ class Music(_VoiceCommon):
     def play(self, lessonfile_ref, question):
         assert isinstance(lessonfile_ref, LessonfileCommon)
         if len(lessonfile_ref.prepare_instrument_list(question)) == 2:
-            _VoiceCommon.play(self, lessonfile_ref, question)
+            MpdParsable.play(self, lessonfile_ref, question)
         else:
             self.play_3patches(lessonfile_ref, question, 
                     lessonfile_ref.get_tempo())
     def play_slowly(self, lessonfile_ref, question):
         assert isinstance(lessonfile_ref, LessonfileCommon)
         if len(lessonfile_ref.prepare_instrument_list(question)) == 2:
-            _VoiceCommon.play_slowly(self, lessonfile_ref, question)
+            MpdParsable.play_slowly(self, lessonfile_ref, question)
         else:
             tempo = lessonfile_ref.get_tempo()
             tempo = (tempo[0]/2, tempo[1])
@@ -614,20 +644,28 @@ class Music(_VoiceCommon):
         Play the music with different instrument for the top and bottom voice.
         Will use the instruments defined in the preferences window.
         """
-        instrument = lessonfile_ref.prepare_instrument_list(question)
-        if len(instrument) == 2:
-            instrument = instrument * 3
-        # FIXME, only the velocity from instrument[1] is used since
-        # music_to_tracklist is not complete.
-        tracklist = mpd.music_to_tracklist(self.get_mpd_music_string(lessonfile_ref), instrument[1])
-        tracklist[0].prepend_patch(instrument[4])
-        a, b = tempo
-        # FIXME the bpm api is wrong. We set the tempo to one track, and it works for all.
-        tracklist[0].prepend_bpm(a, b)
-        for track in tracklist[:-1][1:]:
-            track.prepend_patch(instrument[2])
-        tracklist[-1].prepend_patch(instrument[0])
-        soundcard.synth.play_track(*tracklist)
+        mpd.play_music3(
+            self.get_mpd_music_string(lessonfile_ref),
+            tempo,
+            lessonfile_ref.prepare_instrument_list(question))
+    def get_err_context(self, exception, lessonfile_ref):
+        """
+        Return a twoline string showing what caused the exception.
+        """
+        first = exception.m_linepos1
+        last = exception.m_linepos2
+        if lessonfile_ref.header.random_transpose:
+            s = self.m_musicdata.replace(r'\staff',
+               r'\staff\transpose %s' % lessonfile_ref.m_transpose.get_octave_notename())
+            s = s.replace(r'\addvoice',
+               r'\addvoice\transpose %s' % lessonfile_ref.m_transpose.get_octave_notename())
+            i = len(s) - len(self.m_musicdata)
+        else:
+            i = 0
+        first = exception.m_linepos1 - i
+        last = exception.m_linepos2 - i
+        return "\n".join((self.m_musicdata.split("\n")[exception.m_lineno],
+            " " * first + "^" * (last - first)))
 
 def parse_test_def(s):
     m = re.match("(\d+)\s*x", s)
@@ -713,7 +751,7 @@ class QuestionsLessonfile(LessonfileCommon):
             if self.m_questions[i]['active']:
                 available_question_idx.append(i)
         if not available_question_idx:
-            raise NoQuestionConfiguredException()
+            raise NoQuestionsConfiguredException()
         while 1:
             count += 1
             if cfg.get_bool('config/no_random'):
@@ -887,7 +925,6 @@ class QuestionsLessonfile(LessonfileCommon):
         super(QuestionsLessonfile, self).parse_string(s, really_filename)
         if not self.m_questions:
             raise NoQuestionsInFileException(self.m_filename)
-
     def play_question(self, question=None, varname='music'):
         """Play the question. Play the current question if question is none.
         varname is the name of the variable that contains the music.
@@ -897,8 +934,13 @@ class QuestionsLessonfile(LessonfileCommon):
         try:
             question[varname].play(self, question)
         except mpd.MpdException, e:
-            e.m_mpd_varname = varname
-            e.m_mpd_badcode = question[varname].m_musicdata
+            # This code have to be here for code that run m_P.play_question
+            # exception_handled to be able to say which variable has the bug
+            # and show the bad code.
+            if 'm_mpd_varname' not in dir(e):
+                e.m_mpd_varname = varname
+            if 'm_mpd_badcode' not in dir(e):
+                e.m_mpd_badcode = question[varname].get_err_context(e, self)
             raise
     def play_question_slowly(self, question=None, varname='music'):
         if not question:
@@ -1019,7 +1061,10 @@ class DictationLessonfile(QuestionsLessonfile):
     def get_clue_end(self):
         assert self._idx is not None
         if 'clue_end' in self.m_questions[self._idx]:
-            return Rat(*self.m_questions[self._idx]['clue_end'])
+            try:
+                return Rat(*self.m_questions[self._idx]['clue_end'])
+            except TypeError:
+                raise LessonfileException("The 'clue_end' variable was not well formed")
     def get_clue_music(self):
         assert self._idx is not None
         if 'clue_music' in self.m_questions[self._idx]:
@@ -1081,7 +1126,7 @@ class NameIntervalLessonfile(HeaderLessonfile):
                                 mpd.MusicalPitch.new_from_notename("g''")]
         else:
             if len(self.header.tones) != 2:
-                raise LessonfileException("Error in the lesson file '%s':\nThe length of self.header.tones is not 2" % self.m_filename)
+                raise LessonfileParseException("The length of the lesson file header variable 'tones' has to be 2")
             self.header.tones = [mpd.MusicalPitch.new_from_notename(n) for n in self.header.tones]
 
 class IdByNameLessonfile(QuestionsLessonfile, TestSupport):
@@ -1120,10 +1165,10 @@ class IntervalsLessonfile(HeaderLessonfile, TestSupport):
     def next_test_question(self):
         self.m_test_idx += 1
 
-class ChordLessonfile(QuestionsLessonfile):
+class IdPropertyLessonfile(QuestionsLessonfile):
     def parse_string(self, s, really_filename=None):
         """
-        Call ChordLessonfile.parse_string and set the self.m_props dict.
+        Call IdPropertyLessonfile.parse_string and set the self.m_props dict.
         Change some question variables, so that:
 
           inversion = 0
@@ -1132,7 +1177,26 @@ class ChordLessonfile(QuestionsLessonfile):
 
           inversion = _("root position")
         """
-        super(ChordLessonfile, self).parse_string(s, really_filename)
+        super(IdPropertyLessonfile, self).parse_string(s, really_filename)
+        if self.header.flavour == 'chord':
+            if not self.header.new_button_label:
+                self.header.new_button_label = _("_New chord")
+            if not self.header.lesson_heading:
+                self.header.lesson_heading = _("Identify the chord")
+            if not self.header.qprops:
+                self.header.qprops = ['name', 'inversion', 'toptone']
+                self.header.qprop_labels = [
+                             istr.new_translated("Chord type", _("Chord type")),
+                             istr.new_translated("Inversion", _("Inversion")),
+                             istr.new_translated("Toptone", _("Toptone"))]
+        if not self.header.qprops:
+            raise LessonfileParseException(_("Missing qprops variable in the lesson file %s.") % self.m_filename)
+        # These two tests are needed, so we can have qprops and qprop_labels
+        # lists with only one element.
+        if not isinstance(self.header.qprops, list):
+            self.header.qprops = [self.header.qprops]
+        if not isinstance(self.header.qprop_labels, list):
+            self.header.qprop_labels = [self.header.qprop_labels]
         if len(self.header.qprops) != len(self.header.qprop_labels):
             raise LessonfileParseException(_("Error in the lesson file header of \"%(filename)s\". The variables qprops and qprop_labels has to have the same length.") % {'filename': self.m_filename})
         # m_props will be a dict where each key is the property var name.
@@ -1181,6 +1245,17 @@ class ChordLessonfile(QuestionsLessonfile):
                 self.m_questions[idx] = None
         self.m_questions = [q for q in self.m_questions if q is not None]
 
+class ChordLessonfile(IdPropertyLessonfile):
+    def parse_string(self, s, really_filename=None):
+        super(ChordLessonfile, self).parse_string(
+        s.replace("header {",
+                """
+                header {
+                   qprops = "name", "inversion", "toptone"
+                   qprop_labels = _("Chord type"), _("Inversion"), _("Chord type")
+                """),
+        really_filename)
+
 
 class ElembuilderLessonfile(QuestionsLessonfile):
     def parse_string(self, s, really_filename=None):
@@ -1191,6 +1266,9 @@ class ElembuilderLessonfile(QuestionsLessonfile):
 
 class LessonFileManager:
     def __init__(self, debug):
+        # A list of info about lesson files that where discarded
+        # by the lesson file manager
+        self.m_discards = []
         self.parse(debug)
     def parse(self, debug):
         lessonpath = ['lesson-files',
@@ -1212,6 +1290,8 @@ class LessonFileManager:
                     continue
                 # We save the returned lesson_id because we need it below
                 lesson_id = self.parse_into_uiddb(filename)
+                if not lesson_id:
+                    continue
                 # We have to check if the lesson file has changed,
                 # and if it has, then the results has to be deleted.
                 # This because Solfege has no way of knowing what kind
@@ -1229,18 +1309,17 @@ class LessonFileManager:
     def parse_into_uiddb(self, filename):
                 """
                 Returns the lesson_id of the parsed file.
+                Append to .m_discards and return None if there was any errors.
                 """
                 try:
                     f = LessonIdParser(filename)
                     if not f.has_lesson_id():
                         f.add_lesson_id()
                 except IOError, e:
-                    if e.errno == errno.EACCES:
-                        # These are mostly caused by lock files from
-                        # editors, I think. Ignoring this.
-                        pass
-                    else:
-                        print >> sys.stderr, "\nError running add_lesson_id('%s'):" % filename, e
+                    self.m_discards.append({
+                        'filename': filename,
+                        'exception': e,
+                    })
                     return
                 msg = 'Failed to parse the lessonfile "%s". Do not report this as a bug. Fix the file, or delete it.'
                 if os.path.isabs(filename):
@@ -1259,6 +1338,12 @@ class LessonFileManager:
                     return
                 if not 'title' in p.header:
                     p.header['title'] = '@@@NOT TITLE'
+                if 'module' not in p.header:
+                    self.m_discards.append({
+                        'filename': filename,
+                        'reason': 'no module',
+                    })
+                    return
                 n = {
                    'filename': filename,
                    'mtime': os.stat(filename)[stat.ST_MTIME],
