@@ -1,7 +1,6 @@
-# -*- coding: iso-8859-1 -*-
-
+# vim: set fileencoding=iso-8859-1 :
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin ST, Fifth Floor, Boston, MA  02110-1301  USA
 
-solfege_copyright = "Copyright  1999-2006 Tom Cato Amundsen <tca@gnu.org>, and others."
+solfege_copyright = u"Copyright  1999-2008 Tom Cato Amundsen <tca@gnu.org>, and others."
 
 import sys
 import traceback
@@ -177,6 +176,7 @@ import lessonfilegui
 import abstract
 
 import chord
+import idproperty
 import chordvoicing
 import harmonicinterval
 import melodicinterval
@@ -347,7 +347,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         hdlbox = gtk.HandleBox()
         hdlbox.show()
         hdlbox.add(self.g_ui_manager.get_widget('/Menubar'))
-        self._vbox.pack_start(hdlbox)
+        self._vbox.pack_start(hdlbox, False)
         self.m_help_on_current_merge_id = None
     def create_learning_trees_menu(self):
         if self.m_learning_trees_merge_id:
@@ -491,7 +491,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
                 })
             import webbrowser
             webbrowser.open_new("http://www.solfege.org?%s" % params)
-    def display_exception_message(self, exception):
+    def display_exception_message(self, exception, lessonfile=None):
         """Call this function only inside an except clause."""
         from exceptiondialog import ExceptionDialog
         sourcefile, lineno, func, code = traceback.extract_tb(sys.exc_info()[2])[0]
@@ -499,6 +499,8 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         # file name, not open the file.
         sourcefile = sourcefile.decode(sys.getfilesystemencoding(), 'replace')
         m = ExceptionDialog(exception)
+        if lessonfile:
+            m.add_text(_("Please check the lesson file %s.") % lessonfile)
         if sourcefile:
             m.add_text(_('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': sourcefile, 'lineno': lineno})
         if 'm_nonwrapped_text' in dir(exception):
@@ -623,7 +625,7 @@ GNU General Public License for more details.""")
             loc, filename = self.get_list("app/learningtree")
         except ValueError:
             loc, filename = self.default_learning_tree
-        win.load_file2(loc, filename)
+        win.load_file(loc, filename)
         win.show()
     def post_constructor(self):
         global splash_win
@@ -665,6 +667,62 @@ GNU General Public License for more details.""")
         if sys.platform == 'win32' and self.m_app.win32_get_langenviron() != self.get_string('app/lc_messages'):
             gu.dialog_ok(_("Migrated old language setup. You might have to restart the program all translated messages to show up."))
             self.m_app.win32_put_langenviron(self.get_string('app/lc_messages'))
+        if self.m_app.lessonfile_manager.m_discards:
+            if splash_win:
+                splash_win.hide()
+            self.display_discarded_lessonfiles_info()
+            if splash_win:
+                splash_win.show()
+    def display_discarded_lessonfiles_info(self):
+        assert self.m_app.lessonfile_manager.m_discards
+        dlg = gtk.Dialog(_("Discarded lesson files"), buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+        dlg.set_default_response(gtk.RESPONSE_ACCEPT)
+        cbox = gtk.VBox()
+        dlg.vbox.pack_start(cbox)
+        cbox.set_border_width(12)
+        dlg.set_default_size(400, 300)
+        l = gtk.Label(_("The lesson files listed below was discarded by the program, and will not be available in the program until the errors are corrected."))
+        l.set_line_wrap(True)
+        cbox.pack_start(l, False)
+        sc = gtk.ScrolledWindow()
+        sc.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        vbox = gtk.VBox()
+        sc.add_with_viewport(vbox)
+        cbox.pack_start(sc)
+        for idx, info in enumerate(self.m_app.lessonfile_manager.m_discards):
+            sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+            ibox = gtk.VBox()
+            row1 = gtk.HBox()
+            row1.set_spacing(8)
+            ibox.pack_start(row1, False)
+            l = gtk.Label(_("Filename:"))
+            l.set_alignment(1.0, 0.5)
+            sizegroup.add_widget(l)
+            row1.pack_start(gtk.Label(_("Filename:")), False)
+            row1.pack_start(gtk.Label(info['filename']), False)
+            row2 = gtk.HBox()
+            row2.set_spacing(8)
+            ibox.pack_start(row2, False)
+            l = gtk.Label(_("Reason:"))
+            l.set_alignment(1.0, 0.5)
+            sizegroup.add_widget(l)
+            row2.pack_start(l, False)
+            if 'exception' in info:
+                estr = str(info['exception']).decode(sys.getfilesystemencoding(), 'replace')
+                row2.pack_start(gtk.Label(estr), False)
+            elif 'reason' in info:
+                if info['reason'] == 'no module':
+                    msg = _("Missing module declaration")
+                else:
+                    msg = _("Unknown")
+                row2.pack_start(gtk.Label(msg), False)
+            vbox.pack_start(ibox, False)
+            if idx < len(self.m_app.lessonfile_manager.m_discards) - 1:
+                vbox.pack_start(gtk.HSeparator(), False)
+        dlg.show_all()
+        dlg.run()
+        dlg.destroy()
+
     def activate_exercise(self, module, urlobj=None):
         if self.m_viewer:
             self.box_dict[self.m_viewer].hide()
@@ -812,7 +870,10 @@ def start_app(datadir):
         if issubclass(t, lessonfile.LessonfileException):
             w.display_error_message(msg, str(t))
         elif issubclass(t, osutils.ExecutableDoesNotExist):
-            w.display_error_message(msg, str(t))
+            if len(value.args) > 1:
+                w.display_error_message2(value.args[0], "\n".join(value.args[1:]))
+            else:
+                w.display_error_message(msg, str(t))
         else:
             sys.__excepthook__(t, msg, traceback)
     if not options.disable_exception_handler:
