@@ -1,5 +1,5 @@
 # GNU Solfege - free ear training software
-# Copyright (C) 2006, 2007  Tom Cato Amundsen
+# Copyright (C) 2006, 2007, 2008  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -47,7 +47,13 @@ class Teacher(abstract.Teacher):
         x = len(lexer.m_string)
         retval = []
         while lexer.m_idx < x:
-            toc, toc_data = lexer.get()
+            try:
+                toc, toc_data = lexer.get()
+            except mpd.MpdException, e:
+                e.m_lineno, e.m_linepos1, e.m_linepos2 = lexer.get_error_location()
+                e.m_mpd_varname = qvar
+                e.m_mpd_badcode = self.m_P.get_question()[qvar].get_err_context(e, self.m_P)
+                raise
             if is_rhythm == 'rhythm':
                 if toc_data.m_pitch.get_octave_notename() == 'd':
                     continue
@@ -144,6 +150,8 @@ class Gui(abstract.LessonbasedGui):
         self.practise_box.pack_start(self.g_flashbar, False)
     def on_new_question(self, widget=None):
         def exception_cleanup():
+            self.m_t.end_practise()
+            self.g_tap.set_sensitive(False)
             self.g_repeat.set_sensitive(False)
             self.g_give_up.set_sensitive(False)
         g = self.m_t.new_question()
@@ -153,8 +161,6 @@ class Gui(abstract.LessonbasedGui):
             try:
                 self.do_at_question_start_show_play()
             except Exception, e:
-                e.m_mpd_varname = 'music'
-                e.m_mpd_badcode = self.m_t.m_P.get_question().music.m_musicdata
                 if not self.standard_exception_handler(e, exception_cleanup):
                     raise
             else:
@@ -203,17 +209,10 @@ class Gui(abstract.LessonbasedGui):
                     self.m_t.start_tapping()
                     self.g_flashbar.set(self.please_tap_str)
                 self.g_flashbar.flash(msg)
-        except mpd.MpdException, e:
-            # If the 'rhythm' variable is defined in a question, Solfege will
-            # use this variable when comparing the rhythm. But it will still
-            # use the 'music' variable when playing and evt. displaying the
-            # rhythm.
-            if 'rhythm' in self.m_t.m_P.get_question():
-                qvar = 'rhythm'
-            else:
-                qvar = 'music'
-            e.m_mpd_varname = qvar
-            e.m_mpd_badcode = self.m_t.m_P.get_question()[qvar].m_musicdata
+        except Exception, e:
+            # We need to check for exceptions here because for each
+            # tap, solfege will parse a music string and compare the rhythm
+            # the user has tapped so far.
             if not self.standard_exception_handler(e):
                 raise
     def on_start_practise(self):
